import pandas as pd
import numpy as np
import warnings
from sklearn.metrics import confusion_matrix, classification_report
from sklearn.metrics import auc, roc_curve
# from distinctipy import distinctipy
import os as os
from matplotlib import pyplot as plt

class Metrics:
    y_true = []
    y_pred = []
    y_true_names = []
    y_pred_names = []
    name_to_idx = {}
    n_classes = 0
    digits = 4
    fpr = None
    tpr = None
    roc_auc = None

    @staticmethod
    def classification_report(y_true, y_pred) -> None:
        """Prints out the classification report to console.
        """
        print(classification_report(y_true, y_pred, digits = 4))

    @staticmethod   
    def get_metrics_dictionary(y_true, y_pred) -> dict:
        """Returns a dictionary denoting the metrics of the model. 
        It is the accessible version of classification_report()
        
        Returns:
            dict: A dictionary containing the metrics of the model with respect to the classes that it predicts.
        """
        return classification_report(y_true, y_pred, 
                                     digits = 4, 
                                     output_dict = True)

    @classmethod
    def __assert_equal_length(cls) -> None:

        assert (len(cls.y_true) == len(cls.y_pred)), "The length of the true labels is not the same as the pred labels."
    
    @classmethod
    def __check_equal_number_of_classes(cls) -> None:
        if len(set(cls.y_true)) != len(set(cls.y_pred)):
            warnings.warn("WARNING: The number of classes in the true labels and the predicted labels is not the same.")

    @classmethod
    def set_labels(cls, y_true, y_pred) -> None:
        """Set the names of the labels.

        Args:
            y_true (list): A list containing the true labels of the data.
            y_pred (list): A list containing the predicted labels of the data.
        """
        cls.y_true_names = y_true
        cls.y_pred_names = y_pred
        cls.n_classes = len(set(y_true))
        cls.__assert_equal_length()
        cls.__check_equal_number_of_classes()

    @classmethod
    def get_name_to_idx_dict(cls, y_true, y_pred) -> dict:
        cls.set_labels(y_true, y_pred)
        cls.y_true_names = list(set(cls.y_true))
        cls.y_true_names.sort()
        cls.y_pred_names = list(set(cls.y_pred))
        cls.y_pred_names.sort()
        cls.name_to_idx = {}
        cls.name_to_idx = dict(enumerate(cls.y_true_names))
        cls.name_to_idx = dict([(value, key) for key, value in 
                                 cls.name_to_idx.items()])
        return cls.name_to_idx
    
    @classmethod
    def __prettify_confusion_matrix(cls, confusion_matrix) -> str:
        """Prints out the confusion matrix with labels to console.
        Args:
            confusion_matrix: Uses the confusion matrix to format it in a pretty format. 
        Returns:
            str: Returns a pretty printed confusion matrix.
        """
        pandas_cf = pd.DataFrame(confusion_matrix,
                                 index = cls.y_true_names,
                                 columns = cls.y_true_names).round(cls.digits)
        print(pandas_cf)

    @classmethod
    def confusion_matrix(cls, y_true, y_pred) -> None:
        """Prints out the confusion matrix to console.
        """
        cls.set_labels(y_true, y_pred)
        cls.__prettify_confusion_matrix(confusion_matrix(cls.y_true, cls.y_pred))
    
    @classmethod
    def norm_confusion_matrix(cls, along = 'row') -> None:
        """Prints out a normalized confusion matrix to console, either normalized along the rows or columns.

        Args:
            along (str, optional): Takes 'row' or 'col' as arguments. Defaults to 'row'.
            The user specifies if the confusion matrix should be normalized along its rows or columns. 
        """
        AXIS_NUM = 1 if along == 'row' else 0

        DENOMINATOR = (cls.__confusion_matrix.sum(axis = AXIS_NUM)[:, np.newaxis])
        if AXIS_NUM:
            NUMERATOR = cls.__confusion_matrix.astype('float')
            norm_conf_mat = (NUMERATOR / DENOMINATOR)
        else:
            NUMERATOR = cls.__confusion_matrix.T.astype('float')
            norm_conf_mat = (NUMERATOR / DENOMINATOR).T
        rounded_matrix = np.around(norm_conf_mat, decimals = cls.digits)
        cls.__prettify_confusion_matrix(rounded_matrix)

    @classmethod
    def _setup_aucroc(cls, y_softmax_probs: np.array):
        """A protected method to setup the auc-roc plot.

        Args:
            y_softmax_probs (np.array): This is an np.array of the predicted probabilities of the data. The np.array that contains the softmax probs of all classes of all predictions.
        """
        # Much code for this function is adapted from
        # https://scikit-learn.org/stable/auto_examples/model_selection/plot_roc.html
        fpr = dict()
        tpr = dict()
        roc_auc = dict()
        y_true_as_idx = np.array([cls.name_to_idx.get(x) for x in cls.y_true])
        for i in range(cls.n_classes):
            fpr[i], tpr[i], _ = roc_curve(y_true_as_idx == i, y_softmax_probs[:, i])
            roc_auc[i] = auc(fpr[i], tpr[i])
        all_fpr = np.unique(np.concatenate([fpr[i] for i in range(cls.n_classes)]))
        mean_tpr = np.zeros_like(all_fpr)
        for i in range(cls.n_classes):
            mean_tpr += np.interp(all_fpr, fpr[i], tpr[i])
        mean_tpr /= cls.n_classes
        fpr["macro"] = all_fpr
        tpr["macro"] = mean_tpr
        roc_auc["macro"] = auc(fpr["macro"], tpr["macro"])
        cls.fpr = fpr
        cls.tpr = tpr
        cls.roc_auc = roc_auc  

    @staticmethod
    def plot_roc_curve(cls, y_softmax_probs: np.array, save = True,
                       image_filepath = os.getcwd(), 
                       image_filename = "roc_curve", 
                       class_line_width = 1,
                       macro_average_line_width = 3,
                       color_list = []) -> None:
        """This plots the roc curve in a new window and saves it to file if the 
        user so chooses. 

        Args:
            y_softmax_probs (np.array): This is an np.array of the predicted probabilities of the data. The np.array that contains the softmax probs of all classes of all predictions.
            save (bool, optional): Save the generated image to file? Defaults to True.
            image_filepath (str, optional): The path to which the image is saved. Defaults to os.getcwd().
            image_filename (str, optional): The name of the image file. Defaults to "roc_curve".
            class_line_width (int, optional): Controls how wide the lines the multiple classes should be. Defaults to 1.
            macro_average_line_width (int, optional): Controls how wide the dashed line of the macro average of the ROC curves is. Defaults to 3.
            color_list (list, optional): A list of colors. Length should be the same as the number of classes. Defaults to []. If the user doesn't specify a color list, self._distinct_colors will be used, which is automatically generated.
        """
        cls._setup_aucroc(y_softmax_probs)
        # Much code for this function is adapted from
        # https://scikit-learn.org/stable/auto_examples/model_selection/plot_roc.html
        class_line_width = class_line_width
        macro_average_line_width = macro_average_line_width
        plt.figure()
        plt.plot(
            cls.fpr["macro"],
            cls.tpr["macro"],
            label = "Macro-Avg, (AUC = {0:0.3f})".format(cls.roc_auc["macro"]),
            color = "navy",
            linestyle = ":",
            linewidth = macro_average_line_width)
        for i, color in zip(range(len(cls.y_true)), color_list):
            plt.plot(
                cls.fpr[i],
                cls.tpr[i],
                color = color,
                lw = class_line_width,
                label = "{0}, (AUC = {1:0.3f})".format(cls.y_true_names[i], cls.roc_auc[i]))
        plt.plot([0, 1], [0, 1], "k--", lw = class_line_width)
        plt.xlim([0.0, 1.0])
        plt.ylim([0.0, 1.05])
        plt.xlabel("False Positive Rate")
        plt.ylabel("True Positive Rate")
        plt.title("ROC Curves for All Classes")
        plt.legend(loc="center left", 
            bbox_to_anchor=(1, 0.5), 
            ncol = 1, 
            fancybox = True, 
            shadow = True)
        if save:
            plt.savefig(image_filepath + "/" + image_filename + ".png", 
                        bbox_inches='tight')
        plt.show()


from olea.data.dataset import Dataset
from olea.data.dso import DatasetSubmissionObject

from datasets import load_dataset
import pandas as pd


class HateCheck(Dataset) : 

    def __init__(self, split='train') -> None:
        self.dataset_name = 'HateCheck'
        self.URL = 'COLD-team/HateCheck'
        self.description = 'This is the dataset for HateCheck.'
        self.data_columns = ['functionality', 'case_id' , 'test_case' , 'direction' , 
                            'focus_words' , 'focus_lemma']
        self.label_columns = ['label_gold']
        self.label_column ='label_gold'
        self.unique_labels = None
        self.split = 'test'

        self._data = self._load_data()

    def _load_data(self) -> pd.DataFrame:
        print('Loading data...')
        df = load_dataset(self.URL, 
                        use_auth_token='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')[self.split].to_pandas()

        df = df.drop(columns='Unnamed: 0')
        df['target_ident'] = df["target_ident"].astype(str)
        return df 

    def submit(self, dataset: pd.DataFrame, submission: iter, map: dict = None) -> DatasetSubmissionObject:
        submission_df = super().submit(dataset, submission, map)
        return HateCheckSubmissionObject(submission_df)


class HateCheckSubmissionObject(DatasetSubmissionObject) : 

    label_column = 'label_gold'
    prediction_column = 'preds'
    text_column = 'test_case'
    data_columns = ['functionality', 'case_id' , 'test_case' , 'direction' , 
                    'focus_words' , 'focus_lemma']

    def __init__(self, submission:DatasetSubmissionObject):
        self.submission = submission.submission


    def filter_submission(self, on:str, filter:callable, **kwargs):

        self.submission['filter_results'] = self.submission[on].apply(filter)
        filtered_submission = self.submission[self.submission['filter_results'] == True]

        if 'columns' in kwargs : 
            outputs = [filtered_submission[col] for col in kwargs['columns']]
            return outputs
    
        else : 
            return filtered_submission[self.data_columns]
        


if __name__ == '__main__' : 

    hc = HateCheck()
    hc_data = hc.data()

    print(pd.unique(hc._data[hc.label_columns].values.ravel('K')))

    import numpy as np
    mock_preds = np.random.choice([0 , 1] , size=hc_data.shape[0])
    map = {'hateful' : 1 , 'non-hateful' : 0}
    map = {1 : 'hateful', 0 : 'non-hateful'}

    hcso = hc.submit(hc_data, mock_preds, map)
    print(hcso.submission)






from olea.metrics.metrics import Metrics
import numpy as np
import pandas as pd

def get_examples(submission,column,results, sort_list= False):
    """pull examples where model output label does not line up with true label to illustrate specific cases. Adds one text examples from each value present in the specified column to results data

    Args:
        df (df): data to pull from, containing textual data
        column (string): column name to evaluate on
        results (df): df containing information for each classification, (created by all analysis functions)
        sort_list (boolean): if the values need to be sorted for presentation (currently just used by text_length analysis)
    Returns:
        df: updated results 
    """
    df = submission.submission
    off_col = submission.label_column
    preds = submission.prediction_column
    
    column_vals = results[column]
    if sort_list:
        column_vals = sorted(column_vals,key=lambda x: float(x.split('-')[0].replace(',','')))
    examples= ["" for x in range(results.shape[0])]
    examples_pred = ["" for x in range(results.shape[0])]
    examples_gold= ["" for x in range(results.shape[0])]
    
    incorrect_df = df[df[preds] != df[off_col]]
    
    for i in range(len(column_vals)):
        incorrect_values = incorrect_df[incorrect_df[column] == column_vals[i]]
        if incorrect_values.empty:
            examples[i] = ""
            examples_pred[i] = ""
            examples_gold[i] = ""
        else:
            example = incorrect_values.sample(1)
            examples[i] = example[submission.text_column].iloc[0]
            examples_pred[i] = example[submission.prediction_column].iloc[0]
            examples_gold[i] = example[submission.label_column].iloc[0]
        
    results["Example with Incorrect Classification"] = examples
    results["Example Predicted Label"] = examples_pred
    results["Example Gold Label"] = examples_gold
    
    return results
    

def get_metrics(submission, column, cats_based_on_labels = False):

    """Returns metrics information for each categroy specified by column if a column is specified, otherwise it returns metrics over the whole dataset
    
    Args:
        df (df): data to pull from, containing textual data
        column (string): column name to evaluate on, defaults to None. If None, metrics are calculated over the whole dataset
        off_col (string): label of ground truth column
        remove_cat_labels (boolean): if nonsensical labels would be present in results (currently just used by fine-grained category analysis for COLD)
    Returns:
        df: metrics information
    
    """
    off_col = submission.label_column
    preds = submission.prediction_column
    df = submission.submission
    
    if column == off_col:
        #coarse metrics over entire dataset
        # my_metric = Metrics(df[off_col],df["preds"])
        metrics_dict = Metrics.get_metrics_dictionary(y_true = df[off_col], 
        y_pred = df[preds])
        del metrics_dict["accuracy"]
        metrics_df = pd.DataFrame(metrics_dict)
    else:
        #metrics by category specified by column
        metrics_dict = {}
        column_vals = np.unique(df[column].astype(str))
        for value in column_vals:
            df_subset = df.loc[df[column] == value]
            # my_metric = Metrics(df_subset[off_col],df_subset["preds"])
            m_dict = Metrics.get_metrics_dictionary(y_true = df[off_col], 
                                                    y_pred = df[preds])
            del m_dict["accuracy"] #remove accuracy metric, can be viewed elsewhere
            
            if cats_based_on_labels:
                m_dict = __remove_nonsensical_labels(m_dict, df_subset,off_col,np.unique(df[off_col]))
            metrics_dict[value] = m_dict
    
        metrics_df = __generate_metrics_df(metrics_dict,column)
    #return df for dataframe version
    return metrics_df

def __generate_metrics_df(metrics_dict,column):
    """Converts dict of dicts to double-indexed df for better viewing of metrics

    Args:
        metrics_dict (dict): metrics info 
        column (str): Name of Category for Metrics -corresponds to a column name

    Returns:
        df: metrics info in dataframe form
    """
    #Double index version
    reformed_dict = {}
    for outerKey, innerDict in list(metrics_dict.items()):
        for innerKey, values in innerDict.items():
            reformed_dict[(outerKey, innerKey)] = values
    
    multi_indices = pd.MultiIndex.from_tuples(reformed_dict, names=[column, "Metrics"])
    metrics_df = pd.DataFrame.from_dict(reformed_dict.values())
    metrics_df = metrics_df.set_index(multi_indices)
    
    return metrics_df

def __remove_nonsensical_labels(m_dict, df_subset,off_col,off_labels):
    """changes nonsensical metrics label information to dashes. Nonsensical labels arrise when
       in metrics when the labels of the category that is being analyzed are derived partly from the Offensive 
       Label itself. For example in COLD, the fine-grained categorie that include "Reclaimed Slur" is in definition 
       Non-offensive, so this function removes metrics for the category "reclaimed slur" that are offensive. This helps for the
       clarity


    Args:
        m_dict (dict): the metrics dict to remove 
        df_subset (df): subset of df containing only instances of the specific category
        off_col (string): ground truth column name for offensiveness 

    Returns:
        m_dict: m_dict with dashes where nonsensical information would be
    """
    #remove unnecessary labels when getting metrics for Fine-grained categories
    label_to_keep = df_subset[off_col].iloc[0]
    if label_to_keep == off_labels[0]:
        label_to_drop = off_labels[1]
    else:
        label_to_drop = off_labels[0]
    for key in m_dict[label_to_drop]:
        m_dict[label_to_drop][key] = '-'
        
    return m_dict

# def get_plotting_info_create_col(df1,df2, new_feature,off_col):
#     """prepare two dataframes representing (Y/N) binary distinction of a category for metrics and plotting calculation. 
#     Creates a new column of that category for metrics and plotting usage. Passes df with new column into get_plotting_info_from_col 
#     for calculation.

#     Args:
#         df1 (df): one dataframe containing "Y" instances of whatever category. (ie. Instances that contain a hashtag)
#         df2 (df): one dataframe containing "N" non-instances of whatever category.  (ie. Instances that do not contain a hashtag)
#         new_feature (string): name of the new feature (ie. Presence of hashtag)
#         off_col (string): offesnive column

#     Returns:
#        totals  (list): total instances corresponding to each category
#        correct_predictions_n  (list): total instances corresponding to each category that the model predicted correctly
#        results  (df): summarization of results, corresponds to plotted information
#        full_df  (df): df with the new column - necessary for plotting and retrieving examples
#     """
#     df1[new_feature] = "Y"
#     df2[new_feature] = "N"
#     full_df = df1.merge(df2,"outer")

#     totals, correct_predictions_n,results= get_plotting_info_from_col(full_df, new_feature, off_col)

    # totals = [df1.shape[0], df2.shape[0]]
    # #find correct predicitons
    # correct_preds_1 = df1[(df1['preds'] == df1[off_col])]
    # correct_preds_2 = df2[(df2['preds'] ==df2[off_col])]
    # correct_predictions_n = [correct_preds_1.shape[0], correct_preds_2.shape[0]]
    
    
    
    # percents = [a/b if b else 0 for a,b in zip(correct_predictions_n, totals)]
    # results = pd.DataFrame({"Full Agreement" : ["Y","N"],"Total_Correct_Predictions": correct_predictions_n, "Total": totals,"Accuracy": percents})
    
    return totals, correct_predictions_n, results, full_df

def get_plotting_info_from_col(submission, feature):
    """calculates info from a dataframe for metrics and plotting usage using labels from the feature parameter.

    Args:
        df (df): full dataframe with text,off label, and contains a column with category labels for each isntance
        feature (string): name of the feature
        off_col (string): offesnive column

    Returns:
       totals  (list): total instances corresponding to each category
       correct_predictions_n  (list): total instances corresponding to each category that the model predicted correctly
       results  (df): summarization of results, corresponds to plotted information. Contains totals, correct_predictions, and accuracies
    """
    df = submission.submission
    off_col = submission.label_column
    preds = submission.prediction_column
    
    correct_preds = df[(df[preds] == df[off_col])]
    #get totals
    totals = df[feature].value_counts()
    correct_predictions_n = correct_preds[feature].value_counts()
    
    total_df = pd.concat([totals,correct_predictions_n],axis=1)
    totals = total_df.iloc[:,0]
    correct_predictions_n = total_df.iloc[:,1]

    
    Accuracy = correct_preds[feature].value_counts()/df[feature].value_counts()
    results = pd.DataFrame({"Total":totals, "Total_Correct_Predictions": correct_predictions_n, "Accuracy": Accuracy})
    results = results.reset_index()
    results = results.rename(columns = {'index':feature})
    
    return totals, correct_predictions_n,results
    



import numpy as np
import pandas as pd
from olea.data.dataset import Dataset
from olea.data.dso import DatasetSubmissionObject
from datasets import load_dataset

class COLD(Dataset) :

    def __init__(self, split='train') -> None:
        self.dataset_name = 'Complex and Offensive Language Dataset'
        self.URL = 'COLD-team/COLD'
        self.description = 'This is the dataset from COLD.'
        self.data_columns = ['ID', 'DataSet', 'Text']
        self.label_columns = ['Off', 'Slur', 'Nom', 'Dist']
        self.label_column = 'Off'
        self.unique_labels = None
        self.split = split

        self._data = self._load_data()

    def _load_data(self) -> pd.DataFrame:
        print('Loading data...')
        return load_dataset(self.URL,sep='\t')[self.split].to_pandas()

    def submit(self, dataset: pd.DataFrame, submission: iter, map: dict = None) -> DatasetSubmissionObject:
        submission_df = super().submit(dataset, submission, map)
        return COLDSubmissionObject(submission_df)


class COLDSubmissionObject(DatasetSubmissionObject) : 

    label_column = 'Off'
    prediction_column = 'preds'
    text_column = 'Text'
    data_columns = ['Text' , 'Off' , 'Nom' , 'Slur' , 'Dist']
    
    def __init__(self, submission: DatasetSubmissionObject):
        self.submission = submission.submission

    def filter_submission(self, on:str, filter:callable, **kwargs):

        self.submission['filter_results'] = self.submission[on].apply(filter)
        filtered_submission = self.submission[self.submission['filter_results'] == True]

        if 'columns' in kwargs : 
            outputs = [filtered_submission[col] for col in kwargs['columns']]
            return outputs
    
        else : 
            return [filtered_submission['Text'] , filtered_submission['Off'] , filtered_submission['preds']]



if __name__ == '__main__' : 

    cold = COLD()


    print('Testing Generator')

    gen = cold.generator(64)    


    print(next(gen))
    print(next(gen))
    print(next(gen))

    print('Testing submission')

    dataset = next(gen)

    dataset.head()

    num_preds = dataset.shape[0]

    yn_preds = np.random.choice(['Y' , 'N'], size=num_preds)
    bool_preds = np.random.choice([True, False], size=num_preds)

    map = {True : 'Y' , False:'N'}

    print('Yes-No Preds')

    analysis = cold.submit(dataset, yn_preds)



 




from olea.data.dso import DatasetSubmissionObject
from olea.utils.twitteraae.code import detection 
from typing import Union, List
from olea.viz.viz import plot_bar_graph
from olea.viz.viz import plot_histogram, histogram_values
from olea.utils.analysis_tools import get_metrics, get_examples
from olea.utils.analysis_tools import get_plotting_info_from_col
import pandas as pd
import numpy as np

class Generic(object) : 
    
    
    '''TO DO:
        Figure out how to incorporate a list of columns
    '''
    @classmethod
    def _run_analysis_on(cls, submission:DatasetSubmissionObject, 
                        on:Union[str, List[str]], 
                        target_column:str,plot, show_examples) : 

         #labels = np.unique(submission.submission[on])
         totals, correct_predictions_n, results = get_plotting_info_from_col(submission, feature = on)
          
          # plot the bar graph
         if plot:
             plot_bar_graph(totals.index, totals, correct_predictions_n, 
                              title = str("Predictions on " + on))
          #get examples
         if show_examples:
             results = get_examples(submission, on, results)
          
         metrics = get_metrics(submission, on)    
         
         return results, metrics

    @classmethod
    def analyze_on(cls, submission:DatasetSubmissionObject, 
                    features:Union[str, List[str]],plot=True,show_examples=False): 

        return cls._run_analysis_on(submission, 
                                    features, 
                                    submission.label_column, 
                                    plot,
                                    show_examples)

    @classmethod
    def check_substring(cls, submission:DatasetSubmissionObject,
                        substring:str,
                        plot=True,
                        show_examples=False,
                        ):
        """check how model predicts on instances containing a specific substring vs without that substring.

        Args:
            substring (str): stubstring to search for 
            submission (DatasetSUbmissionObject): submission object to use, containing df
            plot (boolean): to plot or not to plot
            show_examples (boolean): to show examples of instances the model predicted incorrectly or not
                                    (WARNING: LIKELY CONTAINS OFFENSIVE LANGUAGE)
            off_col (str): ground truth column name 
        Returns:
            results (df) : results corresponding to plotted information:(total instances for each category,
                             total correctly predicted, and accuracy)
            metrics (df): metrics information for each category
        """ 
        #find instances of substring/ vs no substring
        df1 = submission.submission[submission.submission[submission.text_column].str.contains(substring)]
        df2 = submission.submission[~submission.submission[submission.text_column].str.contains(substring)]
        
        #create new column
        new_feature = ''.join(["Containing ", '\'',substring, '\''])
        labels = [new_feature, str("Not " + new_feature)]
        df1[new_feature] = labels[0]
        df2[new_feature] = labels[1]
        submission.submission = df1.merge(df2,"outer")

        #run analysis
        return cls._run_analysis_on(submission, 
                                    new_feature, 
                                    submission.label_column, 
                                    plot,
                                    show_examples)
        # totals, correct_predictions_n,results= get_plotting_info_from_col(full_df, new_feature, off_col)
        
        
        # plot the bar graph
        # if plot:
        #     plot_bar_graph(labels, totals, correct_predictions_n, 
        #                     title = "Predictions on Text with " + new_feature)
        # #get examples
        # if show_examples == True:
        #     results = get_examples(full_df,new_feature,results,off_col=off_col,sort_list = False)
        
        # #get_metrics
        # metrics = get_metrics(full_df, off_col,new_feature,)
        
        # return results, metrics


    @classmethod
    def aave(cls,submission:DatasetSubmissionObject,
             threshold:float = 0.5,
             plot=True,
             show_examples=False,
             ):
        """Check how model predicts on instances that are written using African American Vernacular English. The scores 
            are calculated using the TwitterAAE model created by (Blodgett et. al 2016). Further information can be found at
            http://slanglab.cs.umass.edu/TwitterAAE/. These scores represent an inference of the *proportion* of words in the 
            text that come from a demographically-associated language/dialect.
        
        Args:
            submission (DatasetSUbmissionObject): submission object to use, containing df
            threshold (float): threshold from (0-1) as cutoff for considering a text to be AAVE
            plot (boolean): to plot or not to plot
            show_examples (boolean): to show examples of instances the model predicted incorrectly or not
                                    (WARNING: LIKELY CONTAINS OFFENSIVE LANGUAGE)
            off_col (str): ground truth column name 
        Returns:
            results (df) : results corresponding to plotted information:(total instances for each category,
                             total correctly predicted, and accuracy)
            metrics (df): metrics information for each category
        """ 
        #get aave values
        aave_values = detection.get_aave_values(submission)
        submission.submission["AAVE"] = aave_values
        df1 = submission.submission[submission.submission['AAVE'] >= threshold]
        df2 = submission.submission[submission.submission['AAVE'] < threshold]
        
        #create new column
        new_feature = "AAVE-thresh >= " + str(threshold) #Add in new feature with labels for metrics
        labels = ["Y", "N"]
        df1[new_feature] = labels[0]
        df2[new_feature] = labels[1]
        submission.submission = df1.merge(df2,"outer")
        
       
        return cls._run_analysis_on(submission, 
                                    new_feature, 
                                    submission.label_column, 
                                    plot,
                                    show_examples)
        
        
        # totals,correct_predictions_n, results, full_df= get_plotting_info_create_col(
        #     data_at,data_bt, new_feature,off_col)

        # # plot the bar graph
        # if plot:
        #     plot_bar_graph(labels, totals, correct_predictions_n, 
        #                     title = "Predictions on Text with " + new_feature)
        # #get examples
        # if show_examples == True:
        #     results = get_examples(full_df,new_feature,results,off_col,sort_list = False)
        
        # #get_metrics
        # metrics = get_metrics(full_df,off_col,new_feature)
        
        # return results, metrics
                

    @staticmethod
    def str_len_analysis(submission:DatasetSubmissionObject,
                         hist_bins = 10,
                         plot=True,
                         show_examples=False,
                         ):
        """Analyze and plot how the model performs on instances of different lengths using a histogram.

        Args:
            submission (DatasetSUbmissionObject): submission object to use, containing df
            hist_bins (int): number of bins to use for the histogram
            plot (boolean): to plot or not to plot
            show_examples (boolean): to show examples of instances the model predicted incorrectly or not
                                    (WARNING: LIKELY CONTAINS OFFENSIVE LANGUAGE)
            off_col (str): ground truth column name 
        Returns:
            results (df) : results corresponding to plotted information:(total instances for each category,
                             total correctly predicted, and accuracy)
            metrics (df): metrics information for each category
        """  
        #add new feature
        new_feature = "Text Length Bin"
        submission.submission[new_feature] = submission.submission[submission.text_column].apply(len)
        correct_preds = submission.submission[(submission.submission[submission.prediction_column] ==submission.submission[submission.label_column])]
        
        if plot:
            plot_histogram(title = "Predictions on Different Text Lengths", 
                           hist_bins = hist_bins,
                           xlabel = "Text Length",
                           list_of_values =  submission.submission[new_feature],
                           correct_preds = correct_preds[new_feature])
        
        bins, bin_vals, bin_vals_correct = histogram_values(submission.submission[new_feature],
                                                correct_preds[new_feature])
        #combine histogram info for printing
        ranges = []
        percents = []
        i = 0
        while i <len(bins)-1:
            ranges.append(str(np.fix(bins[i])) + " - " +str(np.fix(bins[i+1])))
            if bin_vals[i]:
                percents.append(bin_vals_correct[i]/bin_vals[i])
            else:
                percents.append(0)
            i+=1
        
        #reformat bins for clarity
        new_feature_list = []
        for tl in submission.submission[new_feature]:
            for i in range(len(bins)-1):
                #this line looks complicated, but just accounts for the last bin of histogram to be inclusive 
                if (tl>= bins[i] and tl<bins[i+1] or (tl>= bins[i] and tl==(bins[i+ 1]) and i==len(bins)-2)):
                    new_feature_list.append(ranges[i])
        
        submission.submission[new_feature] = new_feature_list
        
        results = pd.DataFrame({new_feature : ranges, "Total": bin_vals, "Total_Correct_Predictions": bin_vals_correct, "Accuracy": percents})
        
        #find examples
        if show_examples:
            results = get_examples(submission, new_feature, results,sort_list = True)

        metrics = get_metrics(submission,submission.label_column,new_feature)
        
        return results,metrics
    
    @classmethod
    def check_anno_agreement(cls,submission: DatasetSubmissionObject, 
                             anno_columns: list,
                             plot = True, show_examples=False) -> pd.DataFrame:
        """This calculates the annotator agreement of the dataset.

        Args:
            submission (DatasetSubmissionObject): This is a DatasetSubmissionObject (which ultimately is a pd.DataFrame)

            anno_columns (list): Give the column names that contain the
            annotator values in the dataframe as a list, 
            off_col (str): the column containing the "ground truth"
            eg: ["Anno1", "Anno2", "Anno3"].
            preds (list): Give the column name that contains the predicted 
            values in the dataframe as a list, eg: ["preds"].
            plot (bool, optional): Plot the annotator agreement as a bar plot. 
            Defaults to True.
            show_examples(bool,optional) : to show examples of instances the model predicted incorrectly or not
                                         (WARNING: LIKELY CONTAINS OFFENSIVE LANGUAGE)

        Returns:
            results(pd.DataFrame): Returns a dataframe that calculates the full 
            agreemeent, total correct predictions, total, and accuracy values 
            for the annotator data.
            metrics (df): metrics information for each category


        """
        #full agreement is considered the "easy case"
        df1 = submission.submission[submission.submission[anno_columns].eq(submission.submission[anno_columns].iloc[:, 0], axis=0).all(axis=1)]
        #not full agreement is considered the more difficult case"
        df2 = submission.submission[~submission.submission.loc[:].isin(df1.loc[:])].dropna()
        
        
        #create new column
        new_feature = "Full Agreement vs Partial Agreement"
        labels = ["Full", "Partial"]
        df1[new_feature] = labels[0]
        df2[new_feature] = labels[1]
        submission.submission = df1.merge(df2,"outer")
        
       
        return cls._run_analysis_on(submission, 
                                    new_feature, 
                                    submission.label_column, 
                                    plot,
                                    show_examples)
        
        # totals,correct_predictions_n, results, full_df= get_plotting_info_create_col(
        #     full_agree,partial_agree, new_feature,off_col)
        
        # # plot the bar graph
        # if plot:
        #     plot_bar_graph(labels, totals, correct_predictions_n, 
        #                     title = "Predictions on Text with" + new_feature) 
        # #get examples
        # if show_examples == True:
        #     results = get_examples(full_df,new_feature,results,off_col=off_col,sort_list = False)
        
        #get_metrics
        # metrics = get_metrics(full_df, off_col,new_feature)
        # return results, metrics
    
    
if __name__ == '__main__' : 

    from olea.data.cold import COLD, COLDSubmissionObject
    from olea.analysis.generic import Generic
    import numpy as np
    from olea.utils.preprocess_text import PreprocessText as pt

    cold = COLD()
    dataset = cold.data()
    messages = list(dataset['Text'])
    pre = pt.execute(messages)
    dataset['pre'] = pre

    num_preds = dataset.shape[0]
    yn_preds = np.random.choice(['Y' , 'N'], size=num_preds)
    bool_preds = np.random.choice([True, False], size=num_preds)

    map = {True : 'Y' , False:'N'}

    print('Yes-No Preds')

    submission = cold.submit(dataset, bool_preds, map=map)

    # coarse_results = Generic.analyze_on(submission,"Off",plot=True,show_examples = True)
    # substr_results = Generic.check_substring(submission,"female",plot=True,show_examples= True)
    # aave_results = Generic.aave(submission,plot = True, show_examples = True)
    # anno_results = Generic.check_anno_agreement(submission, ["Off1","Off2","Off3"],plot = True, show_examples = True)
    str_len_results = Generic.str_len_analysis(submission, plot= True, show_examples = True )
    
   #nom_results = COLDAnalysis.analyze_on(submission,"Nom",plot=True,show_examples = True)
    #cat_results = COLDAnalysis.analyze_on(submission,"Cat", plot= True, show_examples = False)



import pandas as pd

from typing import List, Tuple


class DatasetSubmissionObject(object) : 

    def __init__(self, 
                submission_df:pd.DataFrame, 
                dataset_object:object) : 
        
        for key, value in dataset_object.__dict__.items() : 
            setattr(self, key, value)

        self.submission = submission_df
        self.prediction_column = 'preds'


    def filter_submission(self, on:str, filter:callable, **kwargs)  : 
        pass



# A helper function to preprocess text data

import re
import html
from emoji import demojize
import wordsegment as ws

class PreprocessText:
    messages  = []

    # Preprocessing function
    @classmethod
    def execute(cls, messages: list) -> list:
        """Function to preprocess the messages in COLD according to the
        paper.

        Args:
            messages (list): The column of messages to preprocess.  Must be
            passed as a list.

        Returns:
            list: The preprocessed messages as a list. 
        """
        # This is a pipelined process
        # 1. Lowercase all text, clean unicode characters
        # 2. Replace links with HTML and usernames with USER
        # 3. Segment hashtags into component words
        # 4. Replace emojis with word descriptions
        # 5. Limit consecutive USER mentions to 3 and final cleanup.

        cls.messages = messages

        # 1. Lowercase all text & clearn unicode characters
        preprocess = [x.lower() for x in cls.messages]
        preprocess = [html.unescape(x) for x in preprocess]
        # Replace some characters that can cause issues in other data formats
        # These are typically HTML codes that map to a specific character
        
        # Left double quote to regular double quote:
        preprocess = [x.replace("&#8220;", "\"") for x in preprocess]
        preprocess = [x.replace("“", "\"") for x in preprocess]
        # Right double quote to regular double quote:
        preprocess = [x.replace("&#8221;", "\"") for x in preprocess]
        preprocess = [x.replace("”", "\"") for x in preprocess]
        # Left single quote to regular single quote:
        preprocess = [x.replace("&#8216;", "\'") for x in preprocess]
        preprocess = [x.replace("‘", "\'") for x in preprocess]
        # Right single quote to regular single quote:
        preprocess = [x.replace("&#8217;", "\'") for x in preprocess]
        preprocess = [x.replace("â€¦", "\'") for x in preprocess]
        preprocess = [x.replace("â€™", "\'") for x in preprocess]
        preprocess = [x.replace("’", "\'") for x in preprocess]
        # Ellipses … to triple dot:
        preprocess = [x.replace("&#8230;", "...") for x in preprocess]
        preprocess = [x.replace("…", "...") for x in preprocess]

        # 2. Replace links with HTML and usernames with USER
        # Regex is used to find URLs and USER handles
        preprocess = [re.sub(r"\S*https?:\S*", " HTML ", x) 
                      for x in preprocess]
        preprocess = [re.sub(r"(rt )?@\S*", " USER ", x) for x in preprocess]

        # 3. Segment hashtags into component words
        preprocess = cls.__clean_hashtags(preprocess)

        # 4. Replace emojis with word descriptions
        preprocess = [demojize(x, delimiters = (" ", " ")) for x in preprocess]


        # 5. Limit consecutive USER mentions to 3
        # Trim redundant spaces
        preprocess = [re.sub(' {2,}', ' ', x) for x in preprocess]
        preprocess = [re.sub(r'^\s+', "", x) for x in preprocess]

        preprocess = [re.sub(r'(USER ){4,}', 
                        'USER USER USER ', x) 
                        for x in preprocess]

        # Some messages have </s><s> in them.  Removing them
        # Note from AP, DS: This might represent sarcasm in text.  
        preprocess = [x.replace("</s><s>", " ") for x in preprocess]
        preprocess = [x.replace("</s> <s>", " ") for x in preprocess]

        # Replace _ with a space so that each emoji is converted to its textual
        # tokens. (This is done here to prevent usernames with underscores from
        # earlier in the preprocessing from being wrongly replaced)
        preprocess = [x.replace("_", " ") for x in preprocess]

        # Strip trailing whitespace
        preprocess = [x.strip() for x in preprocess]

        return preprocess

    @classmethod
    def __clean_hashtags(cls, messages: list) -> list:
        """Private helper function to preprocess the messages in COLD 
        which contain a hashtag.

        Args:
            messages (list): The column of messages to preprocess.  Must be
            passed as a list.

        Returns:
            list: The preprocessed messages as a list. 
        """
        
        # The following is adapted from 
        # https://stackoverflow.com/questions/63822966/how-to-use-segment-from-wordsegment-inside-to-re-sub-to-extract-words-from-has
        ws.load()
        hashtag_processed_messages = []
        
        for message in range(len(messages)):
            message_string = messages[message]
            hashtags = re.findall(r"(#\w+)", message_string)

            # If there are hashtags in the message:
            if len(hashtags) != 0: 
                for hashtag in range(len(hashtags)):
                    identified_hashtag = hashtags[hashtag]
                    hashtag_words = " ".join(ws.segment(identified_hashtag))
                    message_string = message_string.replace(identified_hashtag, hashtag_words)
                hashtag_processed_messages.append(message_string)

            # If there are no hashtags in the message, append the message:
            else: 
                hashtag_processed_messages.append(message_string)

        return hashtag_processed_messages


from olea.data.dso import DatasetSubmissionObject
from olea.data.cold import COLDSubmissionObject
from olea.utils.analysis_tools import get_metrics, get_examples
from olea.utils.analysis_tools import get_plotting_info_from_col
from olea.viz.viz import plot_bar_graph
import numpy as np

class COLDAnalysis(object) : 
    
        
    @classmethod
    def _run_analysis_on_functionality(cls,submission:COLDSubmissionObject, on:str,plot, show_examples) :
        cats_based_on_labels =False
        rot = 0
        if on == "Cat":
            cats_based_on_labels =True
            rot = 45
            
        #labels = np.unique(submission.submission[on])
        totals, correct_predictions_n, results = get_plotting_info_from_col(submission, feature = on)
         
         # plot the bar graph
        if plot:
            plot_bar_graph(totals.index, totals, correct_predictions_n, 
                             title = str("Predictions on " + on),rot = rot)
         #get examples
        if show_examples:
            results = get_examples(submission, on, results)
         
        metrics = get_metrics(submission, on,cats_based_on_labels)    
        
        return results, metrics
            
    @classmethod
    def analyze_on(cls, submission:COLDSubmissionObject, on:str,plot=True,show_examples = False) : 
        return cls._run_analysis_on_functionality(submission, on, plot,show_examples)

        

    # @staticmethod
    # def coarse_analysis(cold_submission:DatasetSubmissionObject, off_col = "Off",plot=True,show_examples=False): 
    #     """check how model predicts on instnaces labeled offensive vs non offensive

    #     Args:
    #         cold_submission (DatasetSubmissionObject): submission object to use, containing df
    #         plot (boolean): to plot or not to plot
    #         show_examples (boolean): to show examples of instances the model predicted incorrectly or not
    #                                 (WARNING: LIKELY CONTAINS OFFENSIVE LANGUAGE)
    #         off_col (str): ground truth column name 
    #     Returns:
    #         results (df) : results corresponding to plotted information:(total instances for each category,
    #                          total correctly predicted, and accuracy)
    #         metrics (df): metrics information for each category
    #     """ 
    #     # groundtruth, predictions = cold_submission.submission['Off'] , cold_submission.submission['preds']
    #     labels = np.unique(cold_submission.submission[off_col])
    #     totals, correct_predictions_n, results = get_plotting_info_from_col(cold_submission.submission, off_col, off_col)
        
    #     # plot the bar graph
    #     if plot:
    #         plot_bar_graph(labels, totals, correct_predictions_n, 
    #                         title = "Coarse Predictions of Offensiveness")
    #     #get examples
    #     if show_examples:
    #         results = get_examples(cold_submission.submission,off_col,results,off_col)
        
    #     metrics = get_metrics(cold_submission.submission,column= None, off_col=off_col)    
    #     return results, metrics
    #     # m = Metrics(groundtruth, predictions)
    #     # return m.get_metrics_dictionary()


    # @staticmethod
    # def categorical_analysis(cold_submission:DatasetSubmissionObject, category:str,plot=True,show_examples = False,off_col = "Off",cats_based_on_labels =True) : 
    #     """check how model predicts on different labels of specific category already in the df

    #     Args:
    #         cold_submission (DatasetSubmissionObject): submission object to use, containing df
    #         category (str): column name
    #         plot (boolean): to plot or not to plot
    #         show_examples (boolean): to show examples of instances the model predicted incorrectly or not
    #                                 (WARNING: LIKELY CONTAINS OFFENSIVE LANGUAGE)
    #         off_col (str): ground truth column name 
    #     Returns:
    #         results (df) : results corresponding to plotted information:(total instances for each category,
    #                          total correctly predicted, and accuracy)
    #         metrics (df): metrics information for each category
    #     """ 
    #     totals, correct_predictions_n,results = get_plotting_info_from_col(cold_submission.submission, category, off_col)
    #     labels = np.unique(cold_submission.submission[category])
        
    #     # plot the bar graph
    #     if plot:
    #         plot_bar_graph(labels, totals, correct_predictions_n, 
    #                         title = "Predictions on Text of " + category)
    #     #get examples
    #     if show_examples == True:
    #         results = get_examples(cold_submission.submission,category,results,off_col=off_col)
        
    #     #get_metrics
    #     metrics = get_metrics(cold_submission.submission, category ,off_col,cats_based_on_labels)
        
    #     return results, metrics   
    
    
    
    

        
        # groundtruth, predictions, category_labels = cold_submission.submission['Off'], cold_submission.submission['preds'], cold_submission.submission[category]
    
        # gt_dict = {}
        # pt_dict = {}
        
        # for gt, pt, ct in zip(groundtruth, predictions, category_labels) : 
    
        #     if ct in gt_dict : 
        #         gt_dict[ct].append(gt)
        #         pt_dict[ct].append(pt)
        #     else : 
        #         gt_dict[ct] = [gt]
        #         pt_dict[ct] = [pt]
    
        # result_dict = {}
            
        # for ct in gt_dict.keys() :
        #     m = Metrics(gt_dict[ct] , pt_dict[ct])  
        #     result_dict[ct] = m.get_metrics_dictionary()
                
            
        # return result_dict

if __name__ == '__main__' : 

    from olea.data.cold import COLD, COLDSubmissionObject
    from olea.analysis.cold import COLDAnalysis
    import numpy as np
    from olea.utils.preprocess_text import PreprocessText as pt

    cold = COLD()

    dataset = cold.data()
    messages = list(dataset['Text'])
    pre = pt.execute(messages)
    dataset['pre'] = pre

    num_preds = dataset.shape[0]
    yn_preds = np.random.choice(['Y' , 'N'], size=num_preds)
    bool_preds = np.random.choice([True, False], size=num_preds)

    map = {True : 'Y' , False:'N'}

    print('Yes-No Preds')

    submission = cold.submit(dataset, bool_preds, map=map)

    coarse_results = COLDAnalysis.analyze_on(submission,"Off",plot=True,show_examples = True)
    nom_results = COLDAnalysis.analyze_on(submission,"Nom",plot=True,show_examples = True)
    cat_results = COLDAnalysis.analyze_on(submission,"Cat", plot= True, show_examples = False)

    #print(COLDAnalysis.categorical_analysis(submission, category='Nom'))
















from olea.analysis.generic import Generic
from olea.analysis.cold import COLDAnalysis
from olea.analysis.hatecheck import HateCheckAnalysis

__all__ = [
    "Generic" , 
    "COLDAnalysis" , 
    "HateCheckAnalysis"
]


import matplotlib.pyplot as plt
import numpy as np


def plot_bar_graph(labels, totals, correct_predictions,
                   title, rot = 0, xlabel = ""):
    """Plots bar graph for different analyses

    Args:
        labels (list): labels to be used for x axis
        totals (list): number of total instances for each category
        correct_predictions (list): number of correctly predicted instances for each category
        title (str): title of graph
        rot (int): rotation for x-axis ticks
        xlabel (str): x label
    """
    ha = "center"
    fsize=10
    if len(labels) >2:
        rot = 45
        ha = "right"
        if len(labels) >12:
            fsize= 8
        
    plt.bar(labels, totals, color = "red", label = "Total", edgecolor='black')
    ax = plt.bar(labels, correct_predictions, color = "blue", 
                 label = "Correct Predicitons", edgecolor = 'black')
    plt.legend()
    plt.title(title)
    plt.xticks(ticks = np.arange(labels.shape[0]), labels = labels, rotation = rot,ha=ha, rotation_mode='anchor')
    plt.tick_params(axis='x', labelsize=fsize)
    plt.xlabel(xlabel)
    plt.ylabel("Number of Instances")
    plt.show()

def plot_histogram(hist_bins = 10,legend_location = 'upper right', title = "", 
                   xlabel = "", ylabel = "Num. of Instances", 
                   list_of_values = [], 
                   correct_preds = [],
                   ):
    _, bins, _ = plt.hist(list_of_values,bins=hist_bins, color="red", 
                        label = "Total",edgecolor='black')
    _, _, _ = plt.hist(correct_preds,bins = bins, 
                        color="blue", 
                        label = "Correct Prediciton", 
                        edgecolor='black')
    plt.legend(loc = legend_location)
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.title(title)
    plt.xticks(bins)
    plt.show()

def histogram_values(list_of_values = [], 
                     correct_preds = []) -> list:
    bin_vals, bins, _ = plt.hist(list_of_values, color="red", 
                                 label = "Total",edgecolor='black')
    bin_vals_correct, _, _ = plt.hist(correct_preds,bins = bins, 
                                      color="blue", 
                                      label = "Correct Prediciton", 
                                      edgecolor='black')
    plt.close()
    return bins, bin_vals, bin_vals_correct


from setuptools import setup, find_packages

VERSION = '0.0.5' 
AUTHORS = 'OLEA Team, Anonymized for Review'
DESCRIPTION = 'Short OLEA description'
LONG_DESCRIPTION = 'Longer OLEA description'

print('Finding packages...')
print(find_packages(where='olea' , 
                    exclude=['unittests*' , 'experiments*']))

# find_packages(where='src' , 
#                                 exclude=['unittests*' , 'experiments*'])

# Setting up
setup(
        name = "olea", 
        version = VERSION,
        author = AUTHORS,
        author_email = "<olea.ask@gmail.com>",
        description = DESCRIPTION,
        long_description = LONG_DESCRIPTION,
        packages = ['olea' , 'olea.analysis' , 'olea.metrics' , 'olea.data' , 'olea.utils' , 'olea.viz'],
        install_requires = ['numpy>1.21.0' , 
                            'scipy>1.6.0' , 
                            'datasets>2.2.0' , 
                            'matplotlib>3.0' , 
                            'pandas>1.2.0' , 
                            'Pillow>8.0.0' , 
                            'scikit-learn>1.0' , 
                            'emoji>1.0',
                            'wordsegment>1.3'
                            ], 
        
        keywords = ['python' , 
                    'linguistics',
                    'linguistic-analysis'],
        
        classifiers = [
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Education",
            "Programming Language :: Python :: 3",
            "Operating System :: MacOS :: MacOS X",
            "Operating System :: Microsoft :: Windows",
        ]
)




from olea.data.cold import COLD , COLDSubmissionObject
from olea.data.dataset import Dataset
from olea.data.dso import DatasetSubmissionObject
from olea.data.hatecheck import HateCheck , HateCheckSubmissionObject
from olea.data.hatexplain import HateXplain , HateXplainSubmissionObject

__all__ = [
    
    "Dataset", "DatasetSubmissionObject",
    "COLD" , "COLDSubmissionObject" , 
    "HateCheck" , "HateCheckSubmissionObject" ,
    "HateXplain" , "HateXplainSubmissionObject"
]





from typing import List, Union
import pandas as pd
import numpy as np

from olea.data.dso import DatasetSubmissionObject

class Dataset(object) : 

    def __init__(self, data:pd.DataFrame=None, 
                data_columns=Union[str, List[str]], 
                label_column=str,
                text_column=str) -> None:
        """Initializes the dataset instance. 

        The requested dataset is downloaded and saved on the local system. You can choose where the dataset 
        should be saved by providing a dataset_save_dir.

        Args:
            dataset_save_dir (str, optional): _description_. Defaults to '~/cold/'.
        """
        self.URL = None
        self.dataset_name = None
        self.shape = None
        self.description = None
        self.data_columns = None
        self.label_column = None
        self.text_column = None
        self.unique_labels = None
        self._data = None
        
        self._load_data(data, data_columns, label_column, text_column)

    def __call__(self) -> None:
        pass

    def __iter__(self) -> None: 
        pass

    def __str__(self) -> str:
        pass

    def _load_data(self, data:pd.DataFrame, 
                data_columns:Union[str, List[str]], 
                label_column:str,
                text_column:str) -> None:
        self.data_columns = data_columns
        self.label_column = label_column
        self.text_column = text_column
        self._data = data

    def _find_unique_labels(self) :
        self.unique_labels = pd.unique(self._data[self.label_column].values.ravel('K'))

    def _validate_predictions(self, dataset, submission, map=None) :

        if type(submission) not in [list, np.ndarray, pd.Series, tuple] : 
            raise TypeError('Expected submission to be in the form of one of the following types : '
            'list, np.ndarray, pd.Series, tuple')
        if map is None : 
            predictions = submission
        else : 
            predictions = self._map_submission_to_predictions(submission, map)

        if len(dataset) != len(predictions) : 
            raise ValueError('There is a mismatch in dataset of len() and prediction of len()'.format(len(dataset) , len(predictions)))

        if self.unique_labels is None : 
            self._find_unique_labels()

        for prediction in predictions : 
            if prediction not in self.unique_labels : 
                raise ValueError('You have submitted a prediction that does not have a basis in grountruth. Please provide a mapping.')

        return predictions


    def _map_submission_to_predictions(self, submission, map) :

        if self.unique_labels is None : 
            self._find_unique_labels() 

        if not all(sub in map.keys() for sub in submission) :
            raise ValueError('Problem encountered with one or more of your submissions.'
            'Please check if all submissions has a mapping.')

        if not all(label in self.unique_labels for label in map.values()) :
            raise ValueError('One or more of the mappings provided couldnt be traced back to a '
            'label in the groundtruth.')

        return [map[sub] for sub in submission]

    def data(self) : 
        return self._data[self.data_columns]

    def generator(self, batch_size) -> pd.DataFrame:
        
        batch_size -= 1 # Pandas indexes weird
        start = 0
        end = batch_size

        while True : 

            if end < self._data.shape[0] :

                batch = self._data.loc[start:end, self.data_columns]
                start +=  batch_size + 1 # Once again, Pandas indexes weirdly
                end  = start + batch_size

            elif end >= self._data.shape[0] : 
                start, end = self._data.shape[0] - batch_size, self._data.shape[0]
                batch = self._data.loc[start:end, self.data_columns]
                start, end = 0, batch_size
                
            yield batch

    def info(self) -> None : 
        pass

    def submit(self, dataset:pd.DataFrame, submission:iter, map:dict=None) -> None : 

        valid_predictions = self._validate_predictions(dataset, submission, map)
        submission_df = self._data.loc[self._data.index.isin(dataset.index.values)]
        submission_df['preds'] = valid_predictions

        submission_object = DatasetSubmissionObject(submission_df, self)

        return submission_object




# TODO: This file is scheduled to be deleted soon. 


# import numpy as np
# import pandas as pd
# # import matplotlib.pyplot as plt
# from src.metrics.metrics import Metrics
# from src.viz.viz import plot_bar_graph, plot_histogram, histogram_values


# class Analysis:
#     def __init__(self,cold,show_examples):
#         """Initialize analysis class

#         Args:
#             cold (dataframe): cold data including comun "pred" (predicted labels for each entry in form of 0/1)
#             show_examples (boolean): if textual examples (potentially containing offensive language) should be displayed
#         """
        
#         self.cold = cold
#         self.show_examples = show_examples
#         self.cold_labels = ["Y","N"]

#     def check_string_len(self):
#         """Analyze and plot how the model performs on instances of different lengths using a histogram. Updates cold dataset with new feature columns for text length and text length bin
    
#         Args:
#            self(Analysis): instance of analysis class
        
#         Returns:
#             df: Accuracy of model predictions on different text length ranges 
#             df: multi-index metrics for each class 
#         """
        
#         cold = self.cold
        
#         #update dataframe for this task
#         cold['text_len'] = cold['Text'].apply(len)
#         correct_preds = cold[(cold['pred'] ==cold['OffMaj'])]
        
        
#         #plot histogram
#         plot_histogram(title = "Predictions on Different Text Lengths", 
#                        xlabel = "Text Length",
#                        list_of_values =  cold['text_len'],
#                        correct_preds = correct_preds,)
#         # title = "Predictions on Different Text Lengths"
#         # bin_vals,bins,_ = plt.hist(cold["text_len"],color="red",label = "Total",edgecolor='black')
#         # bin_vals_correct,_,_ = plt.hist(correct_preds["text_len"],bins =bins,color="blue", label ="Correct Prediciton",edgecolor='black')
#         # plt.legend(loc="upper right")
#         # plt.xlabel("Text Length")
#         # plt.title(title)
#         # plt.xticks(bins)
#         # plt.show()

#         bins, bin_vals, bin_vals_correct = histogram_values(cold["text_len"],
#                                                 correct_preds["text_len"])

#         #combine histogram info for printing
#         ranges = []
#         percents = []
#         i = 0
#         while i <len(bins)-1:
#             ranges.append(str(np.fix(bins[i])) + " - " +str(np.fix(bins[i+1])))
#             if bin_vals[i]:
#                 percents.append(bin_vals_correct[i]/bin_vals[i])
#             else:
#                 percents.append(0)
#             i+=1 
            
#         #add new feature
#         new_feature = "Text Length Bin"
#         new_feature_list = []
#         for tl in cold["text_len"]:
#             for i in range(len(bins)-1):
#                 #this line looks complicated, but just accounts for the last bin of histogram to be inclusive 
#                 if (tl>= bins[i] and tl<bins[i+1] or (tl>= bins[i] and tl==(bins[i+ 1]) and i==len(bins)-2)):
#                     new_feature_list.append(ranges[i])
        
#         cold[new_feature] = new_feature_list
        
#         #put histogram info into df
#         results = pd.DataFrame({new_feature : ranges, "Total": bin_vals, "Total_Correct_Predictions": bin_vals_correct, "Accuracy": percents})
        
#         #find examples
#         if self.show_examples:
#              results = self.get_examples(cold,new_feature,results, sort_list = True)
        
#         #combine metrics into one
#         metrics_df = self.get_metrics(new_feature)
#         #results = pd.merge(results, metrics_df,how = "left", right_index = True, left_on = new_feature)
#         return metrics_df, results


#     def check_substring(self,substring):
#         """check how model predicts on instances containing a specific substring vs without that substring. Updates cold dataset with new feature column of contains substring 

#         Args:
#             substring (string): substring to find in instances

#         Returns:
#             df: Accuracy of model predictions on instances containing substring vs not
#             df: multi-index metrics for each class 
#         """
#         cold =self.cold      
#         #find instances of substring/ vs no substring
#         ss = cold[cold['Text'].str.contains(substring)]
#         no_ss = cold[~cold['Text'].str.contains(substring)]
        
#         #Add in new feature
#         new_feature = ''.join(["Contains ", '\'',substring, '\''])
#         ss[new_feature] = "Y"
#         no_ss[new_feature] = "N"
        
#         #find correct predictions on hashtags vs no hashtags
#         correct_preds_ss = ss[(ss['pred'] ==ss['OffMaj'])]
#         correct_preds_nss = no_ss[(no_ss['pred'] ==no_ss['OffMaj'])]
        
#         #find totals
#         labels = ["Y","N"]
#         totals = [ss.shape[0],no_ss.shape[0]]
#         correct_predictions =[correct_preds_ss.shape[0], correct_preds_nss.shape[0]]
        
#         #plot bar graph        
#         title = str("Predictions on Text with " + "'"+ substring+ "'")
#         plot_bar_graph(labels,totals,correct_predictions,title,rot = 0, xlabel= new_feature)
        
#         #create df to store results
#         accuracy = [a/b if b else 0 for a,b in zip(correct_predictions,totals)]
#         results = pd.DataFrame({new_feature : labels,"Total_Correct_Predictions": correct_predictions, "Total": totals,"Accuracy": accuracy})
        
#         #merge all back together and update dataset
#         cold_ss = ss.merge(no_ss,"outer")
#         cold[new_feature] = cold_ss[new_feature]
        
#         if self.show_examples:
#             results = self.get_examples(cold,new_feature,results)
        
#         #combine metrics into one
#         metrics_df = self.get_metrics(new_feature)
#         #results = pd.merge(results, metrics_df, how = "left", right_index = True, left_on = new_feature)
        
#         return results,metrics_df
    
#     def check_confidence(self):
#         pass

#     def fleiss_agreement(self):
#         pass
    
#     def anno_fine_grained(self):
#         """Analyze and plot how the model performs on fine-grained annotator agreement based on how many annotators said the instance was offensive
#         [Y,Y,Y] = 3 , [Y,N,N] = 0 
#         Updates cold dataset with new feature columns for text length and text length bin
    
#         Args:
#            self(Analysis): instance of analysis class
        
#         Returns:
#             df: Accuracy of model predictions on different offesniveness rankings
#             df: multi-index metrics for each class 
#         """
#         cold = self.cold
#         #create annotation rankings
#         rank = []
#         for i in range(len(cold)):
#             annos = [cold.Off1[i],cold.Off2[i],cold.Off3[i]]
#             if all(i == "Y" for i in annos):
#                 rank.append(3)
#             elif all(i == "N" for i in annos):
#                 rank.append(0)
#             elif max(set(annos), key=annos.count) == "Y":
#                 rank.append(2)
#             else:
#                 rank.append(1)
            
                
#         #Add in new feature
#         new_feature = 'Fine-Grained Annotations'
#         cold[new_feature] = rank
        
#         #find correct predictions
#         correct_cold =  cold[(cold['pred'] == cold['OffMaj'])]
        
#         #combine bar chat info into dataframe
#         totals = cold[new_feature].value_counts().sort_index()
#         correct = correct_cold[new_feature].value_counts()
#         Accuracy = correct_cold[new_feature].value_counts()/cold[new_feature].value_counts()
#         results = pd.DataFrame({"Total":totals, "Total_Correct_Predictions": correct, "Accuracy": Accuracy})
#         results = results.reset_index()
#         results = results.rename(columns = {'index':new_feature})
        
        
#         #print examples
#         if self.show_examples:
#             results = self.get_examples(cold,new_feature,results)
        
#         #plot bar graph
#         plot_bar_graph(results[new_feature],results.Total, results["Total_Correct_Predictions"], "Predictions on Fine-Grained Annotations", rot = 0, xlabel = "Number of Annotators that Marked Instance Offesive")
        
#         #combine metrics into one
#         metrics_df = self.get_metrics(new_feature)
#        # results = pd.merge(results, metrics_df, how = "left",right_index = True, left_on = new_feature)
        
#         return results,metrics_df
        
        
    
#     def check_anno_agreement(self,num_annotators):
#         """check how model predicts on instances where annotators fully agree in whether text is offensive ("Y","Y","Y") or ("N","N","N).
#             vs when there is partial agreement. This should indicate performance on "easy" (full) vs "difficult" (partial) cases
#             Updates cold dataset with new feature column of full vs partial agreement

#         Args:
#             num_annotators (int): number of annotators used to determine full vs partial agreement

#         Returns:
#             df: Accuracy of model predictions on full vs partial annotator agreement
#             df: multi-index metrics for each class 
#         """
#         cold = self.cold
         
#         #find indices of full/ partial agreement
#         off_agreements = cold[["Off1", "Off2","Off3"]]
#         #full agreement is considered the "easy case"
#         full_agree = off_agreements[off_agreements.eq(off_agreements.iloc[:, 0], axis=0).all(axis=1)]
#         #not full agreement is considered the more difficult case"
#         partial_agree = off_agreements[~off_agreements.loc[:].isin(full_agree.loc[:])].dropna()
        
#         #include the rest of the cold data
#         full_agree_cold = cold.loc[full_agree.index, :]
#         partial_agree_cold = cold.loc[partial_agree.index,:]
        
#         #Add in new feature 
#         new_feature = "Full Agreement"
#         full_agree_cold[new_feature] = "Y"
#         partial_agree_cold[new_feature] = "N"
        
#         #find correct predicitons
#         correct_preds_full = full_agree_cold[(full_agree_cold['pred'] ==full_agree_cold['OffMaj'])]
#         correct_preds_partial = partial_agree_cold[(partial_agree_cold['pred'] ==partial_agree_cold['OffMaj'])]
        
#         #find totals
#         labels = ["Full","Partial"]
#         totals = [full_agree_cold.shape[0],partial_agree_cold.shape[0]]
#         correct_predictions =[correct_preds_full.shape[0], correct_preds_partial.shape[0]]
        
#         #plot bar graph
#         title = "Predictions on Text with Full vs Partial Annotator Agreement"
#         plot_bar_graph(labels,totals,correct_predictions,title)
        
#         #create df to store results
#         percents = [a/b if b else 0 for a,b in zip(correct_predictions,totals)]
#         results = pd.DataFrame({new_feature : ["Y","N"],"Total_Correct_Predictions": correct_predictions, "Total": totals,"Accuracy": percents})
        
#         #merge all back together
#         cold_agree = full_agree_cold.merge(partial_agree_cold,"outer")
#         cold[new_feature] = cold_agree[new_feature]
        
#         if self.show_examples:
#              results = self.get_examples(cold_agree,new_feature,results)
        
#         #combine metrics into one
#         metrics_df = self.get_metrics(new_feature)
#         #results = pd.merge(results, metrics_df, how = "left", right_index = True, left_on = new_feature)
        
#         return results,metrics_df
    
    
#     def category_performance(self):
#         """check how model predicts on the 10 different fine-gained subcategories described in COLD (Palmer et al, 2020)
#             Updates cold dataset with new feature column of category for each instance

#         Args:
#             num_annotators (int): number of annotators used to determine full vs partial agreement

#         Returns:
#             df: Accuracy of model predictions on full vs partial annotator agreement
#             df: multi-index metrics for each class 
#         """
#         cold =self.cold 
        
#         #create list of categories
#         cats = []
#         for i in range(len(cold)):
#             if cold.OffMaj[i] == "Y":
#                 if cold.SlurMaj[i] == "Y":
#                     cats.append("offSlur")
#                 elif cold.NomMaj[i] == "Y" and cold.DistMaj[i] == "Y":
#                     cats.append("offBoth")
#                 elif cold.DistMaj[i] == "Y":
#                     cats.append("offDist")
#                 elif cold.NomMaj[i] == "Y":
#                     cats.append("offNom")
#                 else:
#                     cats.append("offOther")
#             else:
#                 if cold.SlurMaj[i] == "Y":
#                     cats.append("reclaimed")
#                 elif cold.NomMaj[i] == "Y" and cold.DistMaj[i] == "Y":
#                     cats.append("nonBoth")
#                 elif cold.DistMaj[i] == "Y":
#                     cats.append("nonDist")
#                 elif cold.NomMaj[i] == "Y":
#                     cats.append("nonNom")
#                 else:
#                     cats.append("nonNone")
        
#         #Add in new feature
#         new_feature = 'Category'
#         cold['Category'] = cats
        
#         # #find correct predictions on hashtags vs no hashtags
#         correct_cold =  cold[(cold['pred'] == cold['OffMaj'])]
        
#         #combine bar chat info into dataframe
#         totals = cold[new_feature].value_counts()
#         correct = correct_cold[new_feature].value_counts()
#         Accuracy = correct_cold[new_feature].value_counts()/cold[new_feature].value_counts()
#         results = pd.DataFrame({"Total":totals, "Total_Correct_Predictions": correct, "Accuracy": Accuracy})
#         results = results.reset_index()
#         results = results.rename(columns = {'index':new_feature})
        
#         if self.show_examples:
#             results = self.get_examples(cold,new_feature,results)
        
#         #plot bar chart
#         plot_bar_graph(results[new_feature],results.Total, results["Total_Correct_Predictions"], "Predictions on Fine-Grained Categories",rot = 45,xlabel = "Category")
        
#         #combine metrics into one
#         metrics_df = self.get_metrics(new_feature,True)
#         #results = pd.merge(results, metrics_df,how = "left", right_index = True, left_on = new_feature)
        
#         return results,metrics_df
    
#     # def plot_bar_graph(self,labels, totals, correct_predictions,title,rot = 0,xlabel=""):
#     #     """plots bar graph for different analyses

#     #     Args:
#     #         labels (list): labels to be used for x axis
#     #         totals (list: number of total instances for each class
#     #         correct_predictions (list): number of correctly predicted instances for each class
#     #         title (str): title of graph
#     #         rot (int): rotation for x-axis ticks
#     #         xlabel (str): x label
#     #     """
#     #     plt.bar(labels,totals, color="red",label = "Total",edgecolor='black')
#     #     ax = plt.bar(labels,correct_predictions,color="blue", label ="Correct Predicitons",edgecolor='black')
#     #     plt.legend()
#     #     plt.title(title)
#     #     plt.xticks(ticks = labels, rotation = rot)
#     #     plt.xlabel(xlabel)
#     #     plt.ylabel("Number of Instances")
#     #     plt.show()
        
#     def get_examples(self,df,column,results, sort_list= False):
#         """pull examples where model output label does not line up with true label to illustrate specific cases. Adds one text examples from each value present in the specified column to results data

#         Args:
#             df (df): data to pull from, containing textual data
#             column (string): column name to evaluate on
#             results (df): df containing information for each classification, (created by all analysis functions)
#             sort_list (boolean): if the values need to be sorted for presentation (currently just used by text_length analysis)
#         Returns:
#             df: updated results 
#         """
#         column_vals = np.unique(df[column])
#         if sort_list:
#             column_vals = sorted(column_vals,key=lambda x: float(x.split('-')[0].replace(',','')))
#         examples= ["" for x in range(results.shape[0])]
        
#         incorrect_df = df[df["pred"] != df["OffMaj"]]
        
#         for i in column_vals:
#             results_i = results.index[results[column] == i][0] #get the results index
#             example = np.random.choice(incorrect_df[incorrect_df[column] == i]["Text"], 1)
#             examples[results_i] = example[0]
            
#         results["Example with Incorrect Classification"] = examples
        
#         return results
        
#     def get_metrics(self, column, categories = False):
#      #   df = pd.DataFrame(columns = [column,"Metrics"])
#         column_vals = np.unique(self.cold[column])
#         metrics_dict = {}
#         for value in column_vals:
#             cold_subset = self.cold.loc[self.cold[column] == value]
#             my_metric = Metrics(cold_subset["OffMaj"],cold_subset["pred"])
#             m_dict = my_metric.get_metrics_dictionary()
#             del m_dict["accuracy"] #remove accuracy metric, can be viewed elsewhere
            
#             if categories:
#                 #remove unnecessary labels when getting metrics for Fine-grained categories
#                 label_to_keep = cold_subset["OffMaj"].iloc[0]
#                 if label_to_keep == self.cold_labels[0]:
#                     label_to_drop = self.cold_labels[1]
#                 else:
#                     label_to_drop = self.cold_labels[0]
                
#                 for key in m_dict[label_to_drop]:
#                     m_dict[label_to_drop][key] = '-'
#                 pass
            
#            # df = df.append({column: value, "Metrics" : pd.DataFrame(m_dict)}, ignore_index = True)
#             metrics_dict[value] = m_dict
        
        
#         #Double index version
#         reformed_dict = {}
#         for outerKey, innerDict in metrics_dict.items():
#             for innerKey, values in innerDict.items():
#                 reformed_dict[(outerKey, innerKey)] = values
#         metrics_df = pd.DataFrame(reformed_dict)
        
#         multi_indices = pd.MultiIndex.from_tuples(reformed_dict, names=[column, "Metrics"])
#         full_df = pd.DataFrame.from_dict(reformed_dict.values())
#         full_df = full_df.set_index(multi_indices)
        
#         #return df for dataframe version
#         return full_df
        
    
        

        
    
    


    



import pandas as pd

from olea.data.dataset import Dataset
from olea.data.dso import DatasetSubmissionObject

from datasets import load_dataset

class HateXplain(Dataset) : 

    def __init__(self, split='train') -> None:
        self.dataset_name = 'HateXplain'
        self.URL = 'hatexplain'
        self.description = 'This is the dataset for HateXplain.'
        self.data_columns = ['ID', 'post_tokens']
        self.label_columns = ['label1' , 'label2' , 'label3']
        self.unique_labels = None
        self.split = split

        self._data = self._load_data()

    def _load_data(self) -> pd.DataFrame:
        print('Loading data...')
        df = load_dataset(self.URL)[self.split].to_pandas()

        labels = []
        annotator_ids = []
        targets = []
        rationales = []
        
        for i , row in df.iterrows() : 
            labels.append(row.annotators['label'])
            annotator_ids.append(row.annotators['annotator_id'])
            targets.append(row.annotators['target'])

        df[['label1' , 'label2' , 'label3']] = labels
        df[['anno_id1' , 'anno_id2' , 'anno_id3']] = annotator_ids
        df[['target1' , 'target2' , 'target3']] = targets

        return df
            



    def submit(self, dataset: pd.DataFrame, submission: iter, map: dict = None) -> DatasetSubmissionObject:
        submission_df = super().submit(dataset, submission, map)
        return HateXplainSubmissionObject(submission_df)


class HateXplainSubmissionObject(DatasetSubmissionObject) : 

    def __init__(self, submission : DatasetSubmissionObject):
        self.submission = submission.submission


    def filter_submission(self, on:str, filter:callable, **kwargs):

        self.submission['filter_results'] = self.submission[on].apply(filter)
        filtered_submission = self.submission[self.submission['filter_results'] == True]

        if 'columns' in kwargs : 
            outputs = [filtered_submission[col] for col in kwargs['columns']]
            return outputs
    
        else : 
            return [filtered_submission['post_tokens'] , filtered_submission['label1'] , 
                    filtered_submission['label2'] , filtered_submission['label3'] ,
                    filtered_submission['preds']]



    


from typing import List, Union
import numpy as np

from olea.data.hatecheck import HateCheckSubmissionObject
from olea.utils.analysis_tools import get_metrics, get_examples
from olea.data.dso import DatasetSubmissionObject
from olea.analysis.generic import Generic

class HateCheckAnalysis(object) : 

    label_column = 'label_gold'
    prediction_column = 'preds'

    data_columns = ['functionality', 'case_id' , 'test_case' , 'direction' , 
                    'focus_words' , 'focus_lemma']

    functionalities = ['derog_neg_emote_h', 'derog_neg_attrib_h', 'derog_dehum_h', 'derog_impl_h',
                    'threat_dir_h', 'threat_norm_h', 'slur_h', 'slur_homonym_nh',
                    'slur_reclaimed_nh', 'profanity_h', 'profanity_nh', 'ref_subs_clause_h',
                    'ref_subs_sent_h', 'negate_pos_h', 'negate_neg_nh', 'phrase_question_h',
                    'phrase_opinion_h', 'ident_neutral_nh', 'ident_pos_nh', 'counter_quote_nh',
                    'counter_ref_nh', 'target_obj_nh', 'target_indiv_nh', 'target_group_nh',
                    'spell_char_swap_h', 'spell_char_del_h', 'spell_space_del_h',
                    'spell_space_add_h', 'spell_leet_h']

    categories = {'derogation' : ['derog_neg_emote_h', 'derog_neg_attrib_h', 'derog_dehum_h', 'derog_impl_h'], 
                'threats' : ['threat_dir_h', 'threat_norm_h'],
                'slurs':['slur_h', 'slur_homonym_nh', 'slur_reclaimed_nh'],
                'profanity' : ['profanity_h', 'profanity_nh'], 
                'pronoun_references':['ref_subs_clause_h','ref_subs_sent_h'], 
                'negation':['negate_pos_h', 'negate_neg_nh'], 
                'phrasing' : ['phrase_question_h','phrase_opinion_h'], 
                'identity' : ['ident_neutral_nh', 'ident_pos_nh'],
                'counter' : ['counter_quote_nh','counter_ref_nh'],
                'nonhateful-abuse' : ['target_obj_nh', 'target_indiv_nh', 'target_group_nh'],
                'hateful-abuse' : ['spell_char_swap_h', 'spell_char_del_h', 'spell_space_del_h',
                                'spell_space_add_h', 'spell_leet_h']
                }

    @classmethod
    def _run_analysis_on_functionality(cls, submission:HateCheckSubmissionObject, on:Union[str,List[str]],show_examples,plot) :
    
        if type(on) == str and on == "category":
            new_dic = {}
            for k,v in cls.categories.items():
                for x in v:
                    new_dic.setdefault(x,[]).append(k)
                    
            submission.submission[on] = submission.submission.apply (lambda row: new_dic[row.functionality][0], axis=1)
            return Generic.analyze_on(submission, on, show_examples, plot)
        
        
        if type(on) ==str and on in cls.categories : 
            #on is a str of one category
           df1 = submission.submission[submission.submission['functionality'].isin(cls.categories[on])]
           df2 = submission.submission[~submission.submission['functionality'].isin(cls.categories[on])]
           new_feature = on
           #analysis_set = submission.submission[submission.submission['functionality'].isin(cls.categories[on])]
           #find instances of feature vs not feture
           
        if set(on) <= cls.categories.keys():
            #On is a list of categories            
            categories = []
            for f in on:
                for x in cls.categories[f]:
                    categories.append(x)
            
            df1 = submission.submission[submission.submission['functionality'].isin(categories)]
            df2 = submission.submission[~submission.submission['functionality'].isin(categories)]
            new_feature = str(", ").join(on)
            
        labels = [new_feature, str("Not " + new_feature)]
        df1[new_feature] = labels[0]
        df2[new_feature] = labels[1]
        submission.submission = df1.merge(df2,"outer")
        return Generic.analyze_on (submission, new_feature, show_examples, plot)
           
           
        #elif type(on) == str : 
           # analysis_set = submission.submission[submission.submission['functionality'] == on]
        # else : 
        #     analysis_set = submission.submission[submission.submission['functionality'].isin(on)]

        
        #create new column
           

       
        # return get_metrics(df = analysis_set, 
        #                    off_col = cls.label_column, 
        #                    column = None)
    @classmethod
    def analyze_on(cls, hatecheck_submission:HateCheckSubmissionObject, on:Union[str, List[str]], show_examples = True, plot =True) : 
        return cls._run_analysis_on_functionality(hatecheck_submission, on, show_examples, plot)

    # @classmethod
    # def analyze_on_all(cls, hatecheck_submission:HateCheckSubmissionObject) :
    #     return cls._run_analysis_on_functionality(submission=hatecheck_submission.submission, on=cls.functionalities)


if __name__ == '__main__' : 

    import pandas as pd

    from olea.data.hatecheck import HateCheck

    hc = HateCheck()
    hc_data = hc.data()

    import numpy as np
    mock_preds = np.random.choice([0 , 1] , size=hc_data.shape[0])
    map = {1 : 'hateful', 0 : 'non-hateful'}

    hcso = hc.submit(hc_data, mock_preds, map)

    print('Analysis on acceptable abuse...')
    print(Generic.analyze_on(hcso, 'functionality'))

    print('Analysis on hateful abuse...')
    print(HateCheckAnalysis.analyze_on(hcso, ['profanity','threats','slurs']))

    print('Analysis on hateful abuse...')
    print(HateCheckAnalysis.analyze_on(hcso, 'category'))

    print('Analysis on negation...')
    print(HateCheckAnalysis.analyze_on(hcso, 'threats'))



    # print('Analysis on all functionalities...')
    # print(HateCheckAnalysis.analyze_on_all_functionalities(hcso))

    

    

    
    